
module RedPack

  class SimpleHashDispatcher
    include RedPack::MethodDispatcher

    def initialize(dispatch_table, responder)
      @dispatch_table = dispatch_table
      @responder = responder
    end

    def dispatch(ctx, method, params)
      if @dispatch_table.key? method then
        begin
          result = @dispatch_table[method].call(*params)
          @responder.ok(ctx, result)
        rescue
          @responder.fail(ctx, $!)
        end  
      else
        @responder.fail(ctx, "no-such-dispatchable-key: #{method}")
      end  
    end

  end



  class SimpleInstanceDispatcher
    include RedPack::MethodDispatcher

    def initialize(target, responder)
      @target = target
      @responder = responder
    end

    def dispatch(ctx, method, params)
      if @target.respond_to? method then
        begin
          result = @target.send(method, *params)
          @responder.ok(ctx, result)
        rescue
          @responder.fail(ctx, $!)
        end
      else
        @responder.fail(ctx, "no-such-method: #{method} on: #{target}")
      end
    end
  end

 
end

